@@ -2,7 +2,7 @@
 
 if(getRversion() >= '2.5.1') {
   globalVariables(c('demo_info', 'demo_unfiltered', 'demo_filtered', 'demo_meta', 'demo',
-                    'ani_id', 'Animal', 'Date', 'site', 'LocationID', 'tags', 'js', 'DateTime',
+                    'ani_id', 'Animal', 'Date', 'site', 'LocationID', 'tags', 'DateTime',
                     'Elevation', 'TimeDiffMins', 'Rate', 'Longitude', 'Latitude', 'LongBin',
                     'LatBin', 'Duration', 'stopApp', 'Speed', 'Slope', 'Aspect'))
 }
@@ -23,12 +23,20 @@ if(getRversion() >= '2.5.1') {
 #'@export
 #'
 app_server <- function(input, output, session) {
+  # initialize list of datasets
+  meta <- reactiveVal(demo_meta)
+  uploaded <- reactiveVal(FALSE)
+  processingInitiated <- reactiveVal(FALSE)
+  processingInitiatedAll <- reactiveVal(FALSE)
   
   raw_dat <- reactive({
     if(is.null(input$zipInput)) {
       return(demo_info)
     }
-    return(store_batch_list(input$zipInput))
+    dat_info <- store_batch_list(input$zipInput)
+    meta(dat_info$meta)
+    uploaded(TRUE)
+    return(dat_info)
   })
   
   output$numUploaded <- renderText(paste0(ifelse(is.null(input$zipInput), 0, length(raw_dat()$data)), " files uploaded"))
@@ -51,27 +59,26 @@ app_server <- function(input, output, session) {
     }
   })
   
-  # initialize list of datasets
-  meta <- reactiveVal(demo_meta)
-  uploaded <- reactiveVal(FALSE)
-  
   
   observeEvent(input$processButton, {
-    if(!identical(raw_dat(), demo_info)) {
-      uploaded(TRUE)
-      if(!is.null(input$selected_lat) && !is.null(input$selected_long)) {
-        meta(clean_store_batch(raw_dat(), filters = TRUE, zoom = input$selected_zoom,
+      if(!is.null(lat_bounds()) && !is.null(long_bounds())) {
+        processingInitiatedAll(TRUE)
+        meta(clean_store_batch(raw_dat(), filters = input$filterBox, zoom = input$selected_zoom,
                                input$slopeBox, input$aspectBox, 
-                               input$selected_lat[1], input$selected_lat[2],
-                               input$selected_long[1], input$selected_long[2]))
+                               lat_bounds()[1], lat_bounds()[2],
+                               long_bounds()[1], long_bounds()[2]))
       }
       else {
+        processingInitiatedAll(TRUE)
         meta(clean_store_batch(raw_dat(), input$filterBox, zoom = input$selected_zoom,
                                input$slopeBox, input$aspectBox, 
                                raw_dat()$min_lat, raw_dat()$max_lat,
                                raw_dat()$min_long, raw_dat()$max_long))
       }
-    }
+  })
+  
+  observeEvent(input$processSelectedButton, {
+    processingInitiated(TRUE)
   })
   
   ######################################
@@ -80,82 +87,136 @@ app_server <- function(input, output, session) {
   # last data set accessed
   cache <- reactiveVal(list())
   
-  
   # main dynamic data set
   dat_main <- reactive({
-    req(input$selected_ani, input$dates, meta)
-    
-    meta <- meta()
-    
-    if(any(meta$ani_id  %in% input$selected_ani) ){
-      meta <- meta %>%
-        dplyr::filter(ani_id %in% input$selected_ani)
-    }
-    
-    ani_names <- paste(input$selected_ani, collapse = ", ")
-    cache_name <- paste0(ani_names,", ",input$dates[1],"-",input$dates[2])
-    
-    if(!(cache_name %in% names(cache()))) {
-      # if no user provided data, use demo data
-      if(is.null(input$zipInput)) {
-        current_df <- demo %>% dplyr::filter(Animal %in% meta$ani_id,
-                 Date <= input$dates[2],
-                 Date >= input$dates[1])
-        if(nrow(current_df) == 0) {
-          current_df <- demo %>% dplyr::filter(Animal %in% meta$ani_id)
-        }
+    if(is.null(choose_ani()) || is.null(choose_dates()) || is.null(min_time()) || is.null(max_time())) {
+      if(is.null(choose_recent())) {
+        return(demo)
       }
-      # if user provided data, get it
-      else {
-        # temporarily set current_df to cached df to avoid error
-        current_df <- cache()[[1]]
-        if(any(meta$ani_id  %in% input$selected_ani) ){
-          current_df <- get_data_from_meta(meta, input$dates[1], input$dates[2])
-        }
-      }
-     
-      # add LocationID column to the restricted data set
-      current_df <- current_df %>% 
-        dplyr::mutate(LocationID = 1:dplyr::n())
-              
-      # enqueue to cache
-      updated_cache <- cache()
-      updated_cache[[cache_name]] <- list(df = current_df, ani = input$selected_ani, date1 = input$dates[1], date2 = input$dates[2])
-      
-      # dequeue if there are more than 5 dfs 
-      if(length(updated_cache) > 5) {
-        updated_cache <- updated_cache[-1]
-      }
-      cache(updated_cache)
-    }
-    if(is.null(input$selected_recent)) {
       return(cache()[[1]]$df)
     }
     else {
-      return(cache()[[input$selected_recent]]$df)
+    
+      req(meta)
+      
+      meta <- meta()
+      
+      if(any(meta$ani_id  %in% choose_ani()) ){
+        meta <- meta %>%
+          dplyr::filter(ani_id %in% choose_ani())
+      }
+      
+      ani_names <- paste(choose_ani(), collapse = ", ")
+      
+      
+      min_datetime <- lubridate::with_tz(lubridate::ymd_hms(paste(choose_dates()[1], min_time()), tz="UTC", quiet = TRUE), tz="UTC")
+      max_datetime <- lubridate::with_tz(lubridate::ymd_hms(paste(choose_dates()[2], max_time()), tz="UTC", quiet = TRUE), tz="UTC")
+      
+      cache_name <- paste0(ani_names,", ",min_datetime,"-",max_datetime)
+  
+      if( processingInitiatedAll() || processingInitiated() || (uploaded() || !(cache_name %in% names(cache())))) {
+        # if no user provided data, use demo data
+        if(is.null(input$zipInput)) {
+          current_df <- demo %>% dplyr::filter(Animal %in% meta$ani_id,
+                   DateTime <= max_datetime,
+                   DateTime >= min_datetime)
+          if(nrow(current_df) == 0) {
+            current_df <- demo %>% dplyr::filter(Animal %in% meta$ani_id)
+          }
+        }
+        # if user provided data, get it
+        else {
+          # temporarily set current_df to cached df to avoid error
+          current_df <- cache()[[1]]$df
+          if(processingInitiated()) {
+            processingInitiated(FALSE)
+            
+            cache_name <- paste(cache_name, "(processed)")
+            
+            current_df <- cache()[[choose_recent()]]$df %>%
+              dplyr::filter(Latitude >= lat_bounds()[1], Latitude <= lat_bounds()[2], 
+                            Longitude >= long_bounds()[1], Longitude <= long_bounds()[2],
+                            DateTime >= min_datetime, DateTime <= max_datetime) 
+            
+            if(nrow(current_df) == 0) {
+              return(cache()[[choose_recent()]]$df)
+            }
+            current_df <- clean_location_data(current_df, dtype = "", prep = FALSE, filters = input$filterBox) 
+          
+            
+            status_message <- modalDialog(
+              pre(id = "console"),
+              title = "Please Wait...",
+              easyClose = TRUE,
+              footer = NULL
+            )
+            
+            showModal(status_message)
+            
+            withCallingHandlers({
+              shinyjs::html("console", "")
+              current_df <- lookup_elevation_aws(current_df, zoom = input$selected_zoom, get_slope = input$slopeBox, get_aspect = input$aspectBox)
+            },
+            message = function(m) {
+              shinyjs::html(id = "console", html = m$message)
+            })
+           
+            removeModal()  
+          }
+          else if(processingInitiatedAll()) {
+            processingInitiatedAll(FALSE)
+            meta <- meta()
+            ani_names <- paste(meta$ani_id, collapse = ", ")
+            
+            min_datetime <- min(meta$min_date)
+            max_datetime <- max(meta$max_date)
+            
+            cache_name <- paste(paste0(ani_names,", ",min_datetime,"-",max_datetime), "(processed)")
+            current_df <- get_data_from_meta(meta, min_datetime, max_datetime)
+          }
+          else {
+            if(any(meta$ani_id  %in% choose_ani()) ){
+              current_df <- get_data_from_meta(meta, min_datetime, max_datetime)
+            }
+          }
+        }
+       
+        # add LocationID column to the restricted data set
+        current_df <- current_df %>% 
+          dplyr::mutate(LocationID = 1:dplyr::n())
+        
+        
+                
+        # enqueue to cache
+        updated_cache <- cache()
+        updated_cache[[cache_name]] <- list(df = current_df, ani = choose_ani(), date1 = min_datetime, date2 = max_datetime)
+        
+        # dequeue if there are more than 5 dfs 
+        if(length(updated_cache) > 5) {
+          updated_cache <- updated_cache[-1]
+        }
+        cache(updated_cache)
+      }
+      if(is.null(choose_recent())) {
+        return(cache()[[1]]$df)
+      }
+      return(cache()[[choose_recent()]]$df)
     }
   })
   
-  
+  output$nrow_recent <- renderText(paste0(nrow(dat_main()), " rows selected"))
+  output$head_recent <- renderTable(utils::head(dat_main() %>% dplyr::select(Date, Time, Animal, GPS, Latitude, Longitude, Distance, Rate, Course)))
   
   ######################################
   ## DYNAMIC USER INTERFACE
   
   # select lat/long bounds
   
-  output$lat_bounds <- renderUI({
-    if(!input$filterBox) {
-      return()
-    }
-    shinyWidgets::numericRangeInput("selected_lat", "Latitude Range:", value = c(raw_dat()$min_lat, raw_dat()$max_lat))
-  })
+  lat_bounds <- callModule(reactiveRange, 
+                           id = "lat_bounds", type = "latitude", dat = raw_dat)
   
-  output$long_bounds <- renderUI({
-    if(!input$filterBox) {
-      return()
-    }
-    shinyWidgets::numericRangeInput("selected_long", "Longitude Range:", value = c(raw_dat()$min_long, raw_dat()$max_long))
-  })
+  long_bounds <- callModule(reactiveRange, 
+                           id = "long_bounds", type = "longitude", dat = raw_dat)
   
   output$zoom <- renderUI({
     req(input$mainmap_zoom)
@@ -163,107 +224,46 @@ app_server <- function(input, output, session) {
   })
   
   # select data sites
-  output$choose_site <- renderUI({
-    req(meta)
-    
-    meta <- meta()
-    
-    site_choices <- as.list(as.character(unique(meta$site)))
-    
-    shinyWidgets::pickerInput("selected_site", "Select Site(s)",
-                choices = site_choices,
-                selected = site_choices[c(1,2)],
-                multiple = TRUE,
-                inline = FALSE, options = list(`actions-box` = TRUE)
-    ) 
-  }) 
-  
+  choose_site <- callModule(reactivePicker, "choose_site",
+                            type = "site", req_list = list(meta = meta), 
+                            text = "Select Site(s)", min_selected = 1, max_selected = 2, 
+                            multiple = TRUE, options = list(`actions-box` = TRUE))
   
   # select animals
-  output$choose_ani <- renderUI({
-    
-    req(meta, input$selected_site)
-    
-    meta <- meta()
-    
-    if(nrow(meta %>% dplyr::filter(site %in% input$selected_site)) > 0) {
-      meta <- meta %>% dplyr::filter(site %in% input$selected_site) 
-    }
-    
-    ani_choices <- as.list(as.character(unique(meta$ani_id)))
-   
-    shinyWidgets::pickerInput("selected_ani", "Select Animal(s)",
-                choices = ani_choices,
-                selected = ani_choices[1:4],
-                multiple = TRUE, 
-                inline = FALSE, options = list(`actions-box` = TRUE)
-    )
-  })
+  choose_ani <- callModule(reactivePicker, "choose_ani",
+                           type = "ani", req_list = list(meta = meta, selected_site = choose_site),
+                           text = "Select Animal(s)", min_selected = 1, max_selected = 4,
+                           multiple = TRUE, options = list(`actions-box` = TRUE))
   
   # select dates
-  output$choose_dates <- renderUI({
-    
-    req(meta, input$selected_ani)
-    
-    # Get the data set with the appropriate name
-    
-    meta <- meta()
-    
-    if(nrow(meta() %>% dplyr::filter(ani_id %in% input$selected_ani)) > 0) {
-      meta <- meta %>% dplyr::filter(ani_id %in% input$selected_ani)
-    }
-    
-    max_date <- max(as.Date(meta$max_date), na.rm=TRUE)
-    min_date <- min(as.Date(meta$min_date), na.rm=TRUE)
-        
-    sliderInput("dates", "Date Range", min = min_date,
-                max = max_date, value = c(min_date, max_date), step = 1,
-                animate = animationOptions(loop = FALSE, interval = 1000))
-    
-    
-  })
+  choose_dates <- callModule(datePicker, "choose_dates",
+                             req_list = list(meta = meta, selected_ani = choose_ani), text = "Date Range")
   
+  # select time range
+  min_time <- callModule(time, id = "min_time",
+                         type = "min", meta = meta, selected_ani = choose_ani)
+  
+  max_time <- callModule(time, id = "max_time",
+                         type = "max", meta = meta, selected_ani = choose_ani)
   
   # select variables to compute statistics
-  output$choose_cols <- renderUI({
-    req(input$selected_ani) 
-    
-    var_choices <- c( "Elevation", "TimeDiffMins", "Course", "CourseDiff", "Distance", "Rate", "Slope", "Aspect")
-    shinyWidgets::pickerInput("selected_cols", "Choose Variables for Statistics",
-                choices = var_choices,
-                selected = var_choices[c(1,2,3,4)],
-                multiple = TRUE,
-                inline = FALSE, options = list(`actions-box` = TRUE)
-    )
-  })
+  choose_cols <- callModule(staticPicker, "choose_cols",
+                            selected_ani = choose_ani, text = "Choose Variables for Statistics",
+                            choices = c("Elevation", "TimeDiffMins", "Course", "CourseDiff", "Distance", "Rate", "Slope", "Aspect"),
+                            min_selected = 1, max_selected = 4)
   
   # select summary statistics
-  output$choose_stats <- renderUI({
-    req(input$selected_ani)
-    
-    stats_choices <- c("N", "Mean", "SD", "Variance", "Min", "Max", "Range", "IQR",  "Q1", "Median", "Q3" )
-    shinyWidgets::pickerInput("selected_stats", "Choose Summary Statistics",
-                choices = stats_choices,
-                selected = stats_choices[1:6],
-                multiple = TRUE,
-                inline = FALSE, options = list(`actions-box` = TRUE)
-    )
-  })
+  choose_stats <- callModule(staticPicker, "choose_stats",
+                             selected_ani = choose_ani, text = "Choose Summary Statistics",
+                             choices = c("N", "Mean", "SD", "Variance", "Min", "Max", "Range", "IQR",  "Q1", "Median", "Q3"),
+                             min_selected = 1, max_selected = 6)
   
   # select recent data
-  
-  output$choose_recent <- renderUI({
-    req(dat_main)
-    ani_names <- paste(input$selected_ani, collapse = ", ")
-    cache_name <- paste0(ani_names,", ",input$dates[1],"-",input$dates[2])
-    recent_choices <- names(cache())
-    shinyWidgets::pickerInput("selected_recent", "Select Data",
-                choices = recent_choices,
-                selected = cache_name,
-                multiple = FALSE,
-                inline = FALSE
-    )
-  })
+  choose_recent <- callModule(reactivePicker, "choose_recent",
+                              type = "recent", 
+                              req_list = list(dat_main = dat_main, selected_ani = choose_ani, dates = choose_dates, 
+                                              min_time = min_time, max_time = max_time, cache = cache),
+                              text = "Select Data", multiple = FALSE)
   
   # spatial points for maps
   points_main <- reactive({
@@ -283,12 +283,13 @@ app_server <- function(input, output, session) {
    
     # subset data if user has defined selected locations
     if(is.null(selected_locations())){
-      return(dat_main())
+      return(dat_main() %>% dplyr::filter(Latitude != 0 | Longitude != 0))
     }
   
     else{
       return(
         dat_main() %>%
+        dplyr::filter(Latitude != 0 | Longitude != 0) %>% 
         dplyr::filter(LocationID %in% selected_locations())
       )
     }
@@ -354,23 +355,64 @@ app_server <- function(input, output, session) {
   last_locations <- reactiveVal(NULL)
   
   observe({
-    req(points, input$selected_ani)
+    req(points, choose_ani())
     
     pts <- points()
     
-    if (is.null(input$selected_recent)) {
+    if (is.null(choose_recent())) {
       return(leaflet() %>%  # Add tiles
                addTiles(group = "street map"))
     }
     
-    current_anilist <- cache()[[input$selected_recent]]
+    current_anilist <- cache()[[choose_recent()]]
     
     factpal <-
       colorFactor(scales::hue_pal()(length(current_anilist$ani)), current_anilist$ani)
     
     proxy <- leafletProxy("mainmap", session)
+
+    # Add fencing
     
-    if (is.null(last_drawn()) || (!is.null(selected_locations()) & is.null(last_locations())) || (!is.null(selected_locations()) & !identical(last_locations(), selected_locations()) & !identical(last_drawn()$ani, current_anilist))  
+    if(!is.null(input$kmzInput)) {
+      unlink(file.path("temp"), recursive=TRUE)
+      
+      kmz_coords <- maptools::getKMLcoordinates(kmlfile = utils::unzip(zipfile = input$kmzInput$datapath, 
+                                                      exdir = "temp"),
+                                      ignoreAltitude = TRUE)
+      for(kmz_element in kmz_coords) {
+        if(!is.matrix(kmz_element)) {
+          df_point <- data.frame(lng = kmz_element[1], lat = kmz_element[2])
+          proxy %>% 
+            addCircleMarkers(
+              data = df_point,
+              group = "fencing",
+              radius = 4,
+              stroke = FALSE,
+              weight = 3,
+              opacity = .8,
+              fillOpacity = 1,
+              color = "black",
+              fillColor = "black",
+              popup = ~ paste(
+                paste("Lat/Lon:", paste(kmz_element[2], kmz_element[1], sep =
+                                          ", "))
+              )
+            ) 
+        }
+        else if(kmz_element[1, 1] == kmz_element[nrow(kmz_element), 1] &
+                kmz_element[1, 2] == kmz_element[nrow(kmz_element), 2]) {
+          proxy %>% 
+            addPolygons(data = as.data.frame(kmz_element), lng = ~V1, lat = ~V2, group = "fencing")
+        }
+        else {
+          proxy %>% 
+            addPolylines(data = as.data.frame(kmz_element), lng = ~V1, lat = ~V2, group = "fencing")
+        }
+      }
+      unlink(file.path("temp"), recursive=TRUE)
+    }
+    
+    if (grepl("(processed)", choose_recent()) || is.null(last_drawn()) || (!is.null(selected_locations()) & is.null(last_locations())) || (!is.null(selected_locations()) & !identical(last_locations(), selected_locations()) & !identical(last_drawn()$ani, current_anilist))  
          || (!any(current_anilist$ani %in% last_drawn()$ani)) || (identical(last_drawn()$ani, current_anilist$ani) & identical(last_locations(), selected_locations()) & (last_drawn()$date1 != current_anilist$date1 || last_drawn()$date2 != current_anilist$date2))) {
       for(ani in last_drawn()$ani) {
           proxy %>% clearGroup(ani)
@@ -405,7 +447,7 @@ app_server <- function(input, output, session) {
         shinyjs::js$removePolygon()
       }
     } # if closing bracket
-    else if(!identical(last_drawn()$ani, current_anilist$ani)){
+    else if(uploaded() || !identical(last_drawn()$ani, current_anilist$ani)){
       # remove old points
       for(ani in setdiff(last_drawn()$ani, current_anilist$ani)) {
         proxy %>% clearGroup(ani)
@@ -490,83 +532,18 @@ app_server <- function(input, output, session) {
   ######################################
   # DYNAMIC PLOTS PANEL
   ######################################
+  
   # Elevation Line Plot
-  output$plot_elevation_line <- renderPlot({
-   req(dat)
-    
-    # hist(dat()$TimeDiffMin [dat()$TimeDiffMin < 100], main = "Distribution of Time Between GPS Measurements" )
-    ggplot(dat(), aes(x=DateTime, y=Elevation, group=Animal, color=Animal)) + 
-      labs( title = "Elevation Time Series, by Animal",
-            x = "Date",
-            y = "Elevation (meters)") +
-      ylim(1000,2000) + 
-      geom_line(na.rm = TRUE) + 
-      geom_point(na.rm = TRUE) + 
-      theme_minimal()
-  })
+  output$plot_elevation_line <- callModule(reactivePlot, id = "plot_elevation_line", plot_type = "line", dat = dat)
   
   # Sample Rate Histograms
-  output$plot_samplerate_hist <- renderPlot({
-    req(dat)
-    
-    ggplot(dat(), aes(x=TimeDiffMins, fill=Animal))+
-      geom_histogram(  col="White", breaks = seq(0,40, 2)) +
-      facet_wrap(~Animal, ncol=2)+
-      labs( title = "Sample Rate, by GPS Unit" ,
-            x = "Time between GPS Readings (minutes)", 
-            y = "Frequency") + 
-      theme_minimal()
-    
-    
-  })
+  output$plot_samplerate_hist <- callModule(reactivePlot, id = "plot_samplerate_hist", plot_type = "hist", dat = dat)
   
   # Rate by Animal
-
-  output$plot_rate_violin <- renderPlot({
-    req(dat)
-    
-    ggplot(dat() %>% dplyr::filter(Rate < 50), aes(x=Animal, y= Rate, fill=Animal))+
-      geom_violin() + 
-      geom_boxplot(width=.2, outlier.color = NA) +
-      theme_minimal()+
-      labs( title = "Rate of Travel, by GPS Unit" ,
-            x = "Animal", 
-            y = "Rate of Travel (meters/minute)") +
-      theme(axis.text.x = element_text(angle = 45, hjust = 1))
-
-    
-  })
+  output$plot_rate_violin <- callModule(reactivePlot, id = "plot_rate_violin", plot_type = "violin", dat = dat)
   
   # time spent by lat/long
-  output$plot_time_heatmap <- renderPlot({
-    req(dat)
-    
-    dat<- dat()
-    # Heatmap of Time Spent
-    mybreaks <- list(x = round( seq(min(dat$Longitude), max(dat$Longitude), length.out = 10 ),3),
-                     y = round( seq(min(dat$Latitude), max(dat$Latitude), length.out = 10 ),3))
-    ggplot(dat %>% 
-             dplyr::mutate( LongBin = cut_number(Longitude, 100, 
-                                          labels= round( seq(min(Longitude), max(Longitude), length.out = 100 ),3)
-             ),
-             LatBin = cut_number(Latitude, 100, 
-                                 labels=round( seq(min(Latitude), max(Latitude), length.out = 100 ), 3)
-             )) %>%
-             group_by(LongBin, LatBin, Animal) %>%
-             summarize(Duration = sum(TimeDiffMins, na.rm = TRUE)/60), 
-           aes (x = LongBin,  y = LatBin, fill = Duration))+
-      geom_tile()+
-      facet_wrap(~Animal, ncol=2)+
-      labs( title = "Total Time Spent per Location (hours)" ,
-            x = "Longitude", 
-            y = "Latitude")+
-      scale_fill_gradientn(colors = c("white", "green", "red"))+
-      scale_x_discrete( breaks = mybreaks$x) +
-      scale_y_discrete( breaks = mybreaks$y) +
-      coord_equal()+
-      theme_minimal()
-    
-  })
+  output$plot_time_heatmap <- callModule(reactivePlot, id = "plot_time_heatmap", plot_type = "heatmap", dat = dat)
   
   
   ######################################
@@ -574,167 +551,94 @@ app_server <- function(input, output, session) {
   # Summary Statistics
   
   # Time Difference
-  output$timediff_title <- renderUI({
-    if(is.null(input$selected_stats) | is.null(input$selected_cols) | !("TimeDiffMins" %in% input$selected_cols)) 
-      return()
-    h4("Time Difference (minutes) Between GPS Measurements")
-  })
+  timediff_title <- callModule(statsLabel, "timediff_title", 
+                               choose_cols, choose_stats, 
+                               "TimeDiffMins", "Time Difference (minutes) Between GPS Measurements")
   
-  timediff_stats <- reactive({
-    if(!("TimeDiffMins" %in% input$selected_cols) | is.null(input$selected_stats)) 
-      return()
-    
-    summary <- summarise_col(dat(), TimeDiffMins) 
-    subset(summary, select=c("Animal", input$selected_stats))
-    
-  })
   
-  output$timediff <- renderTable(timediff_stats())
+  timediff <- callModule(stats, "timediff", 
+                         choose_cols, choose_stats, 
+                         "TimeDiffMins", TimeDiffMins, dat)
   
   # Elevation
-  output$elevation_title <- renderUI({
-    if(is.null(input$selected_stats) | is.null(input$selected_cols) | !("Elevation" %in% input$selected_cols)) 
-      return()
-    h4("Elevation")
-  })
+  elevation_title <- callModule(statsLabel, "elevation_title", 
+                               choose_cols, choose_stats, 
+                               "Elevation", "Elevation")
   
-  elevation_stats <- reactive({
-    if(!("Elevation" %in% input$selected_cols) | is.null(input$selected_stats)) 
-      return()
-    
-    summary <- summarise_col(dat(), Elevation) 
-    subset(summary, select=c("Animal", input$selected_stats))
-    
-  })
   
-  output$elevation <- renderTable(elevation_stats())
+  elevation <- callModule(stats, "elevation", 
+                         choose_cols, choose_stats, 
+                         "Elevation", Elevation, dat)
   
   # Speed
-  output$speed_title <- renderUI({
-    if(is.null(input$selected_stats) | is.null(input$selected_cols) | !("Speed" %in% input$selected_cols)) 
-      return()
-    h4("Speed")
-  })
+  speed_title <- callModule(statsLabel, "speed_title", 
+                            choose_cols, choose_stats, 
+                            "Speed", "Speed")
   
-  speed_stats <- reactive({
-    if(!("Speed" %in% input$selected_cols) | is.null(input$selected_stats)) 
-      return()
-    
-    summary <- summarise_col(dat(), Speed) 
-    subset(summary, select=c("Animal", input$selected_stats))
-    
-  })
   
-  output$speed <- renderTable(speed_stats())
+  speed <- callModule(stats, "speed", 
+                      choose_cols, choose_stats, 
+                      "Speed", Speed, dat)
   
   # Course
-  output$course_title <- renderUI({
-    if(is.null(input$selected_stats) | is.null(input$selected_cols) | !("Course" %in% input$selected_cols)) 
-      return()
-    h4("Course")
-  })
+  course_title <- callModule(statsLabel, "course_title", 
+                             choose_cols, choose_stats, 
+                             "Course", "Course")
   
-  course_stats <- reactive({
-    if(!("Course" %in% input$selected_cols) | is.null(input$selected_stats)) 
-      return()
-    
-    summary <- summarise_col(dat(), Course) 
-    subset(summary, select=c("Animal", input$selected_stats))
-    
-  })
   
-  output$course <- renderTable(course_stats())
+  course <- callModule(stats, "course", 
+                       choose_cols, choose_stats, 
+                       "Course", Course, dat)
   
   # Course Difference
-  output$coursediff_title <- renderUI({
-    if(is.null(input$selected_stats) | is.null(input$selected_cols) | !("CourseDiff" %in% input$selected_cols)) 
-      return()
-    h4("Course Difference Between GPS Measurements")
-  })
+  coursediff_title <- callModule(statsLabel, "coursediff_title", 
+                             choose_cols, choose_stats, 
+                             "CourseDiff", "Course Difference Between GPS Measurements")
   
-  coursediff_stats <- reactive({
-    if(!("CourseDiff" %in% input$selected_cols) | is.null(input$selected_stats)) 
-      return()
-    
-    summary <- summarise_col(dat(), CourseDiff) 
-    subset(summary, select=c("Animal", input$selected_stats))
-    
-  })
   
-  output$coursediff <- renderTable(coursediff_stats())
+  coursediff <- callModule(stats, "coursediff", 
+                       choose_cols, choose_stats, 
+                       "CourseDiff", CourseDiff, dat)
   
   # Distance
-  output$distance_title <- renderUI({
-    if(is.null(input$selected_stats) | is.null(input$selected_cols) | !("Distance" %in% input$selected_cols)) 
-      return()
-    h4("Distance")
-  })
+  distance_title <- callModule(statsLabel, "distance_title", 
+                             choose_cols, choose_stats, 
+                             "Distance", "Distance")
   
-  distance_stats <- reactive({
-    if(!("Distance" %in% input$selected_cols) | is.null(input$selected_stats)) 
-      return()
-    
-    summary <- summarise_col(dat(), Distance) 
-    subset(summary, select=c("Animal", input$selected_stats))
-    
-  })
   
-  output$distance <- renderTable(distance_stats())
+  distance <- callModule(stats, "distance", 
+                       choose_cols, choose_stats, 
+                       "Distance", Distance, dat)
   
   # Rate
-  output$rate_title <- renderUI({
-    if(is.null(input$selected_stats) | is.null(input$selected_cols) | !("Rate" %in% input$selected_cols)) 
-      return()
-    h4("Rate")
-  })
+  rate_title <- callModule(statsLabel, "rate_title", 
+                             choose_cols, choose_stats, 
+                             "Rate", "Rate")
   
-  rate_stats <- reactive({
-    if(!("Rate" %in% input$selected_cols) | is.null(input$selected_stats)) 
-      return()
-    
-    summary <- summarise_col(dat(), Rate) 
-    subset(summary, select=c("Animal", input$selected_stats))
-    
-  })
-  output$rate <- renderTable(rate_stats())
+  
+  rate <- callModule(stats, "rate", 
+                       choose_cols, choose_stats, 
+                       "Rate", Rate, dat)
   
   # Slope
+  slope_title <- callModule(statsLabel, "slope_title", 
+                             choose_cols, choose_stats, 
+                             "Slope", "Slope")
   
-  output$slope_title <- renderUI({
-    if(is.null(input$selected_stats) | is.null(input$selected_cols) | !("Slope" %in% input$selected_cols) | !("Slope" %in% colnames(dat()))) 
-      return()
-    h4("Slope")
-  })
   
-  slope_stats <- reactive({
-    if(!("Slope" %in% input$selected_cols) | is.null(input$selected_stats) | !("Slope" %in% colnames(dat()))) 
-      return()
-    
-    summary <- summarise_col(dat(), Slope) 
-    subset(summary, select=c("Animal", input$selected_stats))
-    
-  })
-  
-  output$slope <- renderTable(slope_stats())
+  slope <- callModule(stats, "slope", 
+                       choose_cols, choose_stats, 
+                       "Slope", Slope, dat)
   
   # Aspect
+  aspect_title <- callModule(statsLabel, "aspect_title", 
+                             choose_cols, choose_stats, 
+                             "Aspect", "Aspect")
   
-  output$aspect_title <- renderUI({
-    if(is.null(input$selected_stats) | is.null(input$selected_cols) | !("Aspect" %in% input$selected_cols) | !("Aspect" %in% colnames(dat()))) 
-      return()
-    h4("Aspect")
-  })
   
-  aspect_stats <- reactive({
-    if(!("Aspect" %in% input$selected_cols) | is.null(input$selected_stats) | !("Aspect" %in% colnames(dat()))) 
-      return()
-    
-    summary <- summarise_col(dat(), Aspect) 
-    subset(summary, select=c("Animal", input$selected_stats))
-    
-  })
-  
-  output$aspect <- renderTable(aspect_stats())
+  aspect <- callModule(stats, "aspect", 
+                       choose_cols, choose_stats, 
+                       "Aspect", Aspect, dat)
   
   ##############################################################
   # SUBSET DATA VIA MAP
