@@ -14,7 +14,9 @@ if(getRversion() >= '2.5.1') {
                     'avg', 'Data', 'obs', 'Latitude.x', 'Latitude.y', 'Longitude.x',
                     'Longitude.y', 'Distance.x', 'Distance.y', 'Rate.x', 'Rate.y', 'Course.x',
                     'Course.y', 'Elevation.x', 'Elevation.y', 'Slope.x', 'Slope.y',
-                    'cumDist.y', 'Rate.y'))
+                    'cumDist.y', 'Rate.y', 'RateFlag.x', 'RateFlag.y', 'CourseFlag.x', 'CourseFlag.y',
+                    'DistanceFlag.x', 'DistanceFlag.y', 'TotalFlags.x', 'TotalFlags.y',
+                    'DistanceFlag'))
 }
 
 #'
@@ -207,31 +209,47 @@ qqplot_time <- function(rds_path) {
 #'Compares two animal data frames and calculates summary statistics. 
 #'GPS, date, lat, long, course, distance, rate, elevation column names should match. 
 #'
-#'@param correct reference df
-#'@param candidate df to be compared to the reference
+#'@param correct reference data frame
+#'@param candidate data frame to be compared to the reference
+#'@param use_elev logical, whether to include elevation in summary, defaults to True
 #'@param export logical, whether to export summaries to .csv, defaults to False
 #'@param gps_out desired file name of .csv output summary by GPS collar when export is True
 #'@param date_out desired file name of .csv output summary by date when export is True
-#'@return list containing gps_out and date_out as dfs
+#'@return list containing gps_out and date_out as data frames
 #'@examples
 #'# Compare and summarise unfiltered demo cows to filtered 
 #
 #'compare_summarise_data(demo_unfiltered_elev, demo_filtered_elev)
 #'@export
 #'
-compare_summarise_data <- function(correct, candidate, export = FALSE, gps_out = NULL, date_out = NULL) {
-  
-  correct_gps_summary <- correct %>% 
-    summarise_anidf(GPS, Latitude, Longitude, Distance, Course, Rate, Elevation)
+compare_summarise_data <- function(correct, candidate, use_elev = TRUE, export = FALSE, gps_out = NULL, date_out = NULL) {
+  if(use_elev) {
+    correct_gps_summary <- correct %>% 
+      summarise_anidf(GPS, Latitude, Longitude, Distance, Course, Rate, Elevation)
+    
+    correct_date_summary <- correct %>% 
+      summarise_anidf(Date, Latitude, Longitude, Distance, Course, Rate, Elevation)
+    
+    candidate_gps_summary <- candidate %>% 
+      summarise_anidf(GPS, Latitude, Longitude, Distance, Course, Rate, Elevation)
+    
+    candidate_date_summary <- candidate %>% 
+      summarise_anidf(Date, Latitude, Longitude, Distance, Course, Rate, Elevation)
+  }
+  else {
+    correct_gps_summary <- correct %>% 
+      summarise_anidf(GPS, Latitude, Longitude, Distance, Course, Rate, use_elev = FALSE)
+    
+    correct_date_summary <- correct %>% 
+      summarise_anidf(Date, Latitude, Longitude, Distance, Course, Rate, use_elev = FALSE)
+    
+    candidate_gps_summary <- candidate %>% 
+      summarise_anidf(GPS, Latitude, Longitude, Distance, Course, Rate, use_elev = FALSE)
+    
+    candidate_date_summary <- candidate %>% 
+      summarise_anidf(Date, Latitude, Longitude, Distance, Course, Rate, use_elev = FALSE)
+  }
  
-  correct_date_summary <- correct %>% 
-    summarise_anidf(Date, Latitude, Longitude, Distance, Course, Rate, Elevation)
-  
-  candidate_gps_summary <- candidate %>% 
-    summarise_anidf(GPS, Latitude, Longitude, Distance, Course, Rate, Elevation)
-  
-  candidate_date_summary <- candidate %>% 
-    summarise_anidf(Date, Latitude, Longitude, Distance, Course, Rate, Elevation)
   
   gps_summary <- join_summaries(correct_gps_summary, candidate_gps_summary, by_str="GPS")
   date_summary <- join_summaries(correct_date_summary, candidate_date_summary, by_str="Date")
@@ -254,17 +272,18 @@ compare_summarise_data <- function(correct, candidate, export = FALSE, gps_out =
 #'@param dist distance column
 #'@param course course column
 #'@param rate rate column
-#'@param elev elevation column
-#'@param daily whether to group by both GPS and Date for daily summary, defaults to False
-#'@return df of summary statistics for the animal data frame
+#'@param elev elevation column, must be defined when use_elev is true, otherwise NULL
+#'@param use_elev logical, whether to include elevation in summary, defaults to true
+#'@param daily whether to group by both GPS and Date for daily summary, defaults to false
+#'@return data frame of summary statistics for the animal data frame
 #'@examples
 #'# Summary of demo data by date
 #'
-#'summarise_anidf(demo, Date, Latitude, Longitude, Distance, Course, Rate, Elevation, daily = FALSE)
+#'summarise_anidf(demo, Date, Latitude, Longitude, Distance, Course, Rate, Elevation)
 #'
 #'@export
 #'
-summarise_anidf <- function(anidf, by, lat, long, dist, course, rate, elev, daily = FALSE) {
+summarise_anidf <- function(anidf, by, lat, long, dist, course, rate, elev = NULL, use_elev = TRUE, daily = FALSE) {
   if(daily) {
     anidf <- anidf %>% 
       dplyr::group_by(GPS, Date)
@@ -273,7 +292,7 @@ summarise_anidf <- function(anidf, by, lat, long, dist, course, rate, elev, dail
     anidf <- anidf %>% 
       dplyr::group_by({{by}}) 
   }
-  anidf %>% 
+  summary <- anidf %>% 
     dplyr::summarise(n = n(),
                      meanLat = mean({{lat}}),
                      sdLat = stats::sd({{lat}}),
@@ -284,19 +303,27 @@ summarise_anidf <- function(anidf, by, lat, long, dist, course, rate, elev, dail
                      meanCourse = mean({{course}}),
                      sdCourse = stats::sd({{course}}),
                      meanRate = mean({{rate}}),
-                     sdRate = stats::sd({{rate}}),
-                     meanElev = mean({{elev}}),
-                     sdElev = stats::sd({{elev}}))
+                     sdRate = stats::sd({{rate}}))
+  if(use_elev) {
+    if(daily) {
+      summary <- dplyr::full_join(summary, anidf %>% dplyr::summarise(meanElev = mean({{elev}}), sdElev = stats::sd({{elev}})), by = c("GPS", "Date"))
+    }
+    else {
+      summary <- dplyr::full_join(summary, anidf %>% dplyr::summarise(meanElev = mean({{elev}}), sdElev = stats::sd({{elev}})), by = colnames(summary)[1])
+    }
+  }
+  return(summary %>% dplyr::ungroup())
 }
 
 #'
 #'Joins two animal data frame summaries by a column and appends differences
 #'
-#'@param correct_summary summary df of reference dataset, returned by summarise_anidf
-#'@param candidate_summary summary df of dataset to be compared to reference, returned by summarise_anidf
+#'@param correct_summary summary data frame of reference dataset, returned by summarise_anidf
+#'@param candidate_summary summary data frame of dataset to be compared to reference, returned by summarise_anidf
 #'@param by_str column to join by as a string, null if daily=TRUE
 #'@param daily whether to group by both GPS and Date for daily summary, defaults to False
-#'@return df of joined summaries with differences
+#'@param use_elev logical, whether to include elevation in summary, defaults to true
+#'@return data frame of joined summaries with differences
 #'@examples
 #'# Join date summaries of unfiltered and filtered demo data
 
@@ -313,14 +340,15 @@ summarise_anidf <- function(anidf, by, lat, long, dist, course, rate, elev, dail
 #'@export
 #'
 #'
-join_summaries <- function(correct_summary, candidate_summary, by_str, daily = FALSE) {
+join_summaries <- function(correct_summary, candidate_summary, by_str, daily = FALSE, use_elev = TRUE) {
   if(daily) {
-    summary_all <- dplyr::full_join(correct_summary, candidate_summary, by=c("GPS", "Date"))
+    summary_all <- dplyr::full_join(correct_summary, candidate_summary, by = c("GPS", "Date"))
   }
   else {
-    summary_all <- dplyr::full_join(correct_summary, candidate_summary, by=by_str)
+    summary_all <- dplyr::full_join(correct_summary, candidate_summary, by = by_str)
   }
-   summary_all <- summary_all %>% 
+  
+  summary_all <- summary_all %>% 
     # create difference columns
     dplyr::mutate(nDiff = n.x - n.y) %>% 
     dplyr::mutate(meanLatDiff = meanLat.x - meanLat.y) %>% 
@@ -332,44 +360,15 @@ join_summaries <- function(correct_summary, candidate_summary, by_str, daily = F
     dplyr::mutate(meanCourseDiff = meanCourse.x - meanCourse.y) %>% 
     dplyr::mutate(sdCourseDiff = sqrt((sdCourse.x)^2 + (sdCourse.y)^2)) %>% 
     dplyr::mutate(meanRateDiff = meanRate.x - meanRate.y) %>% 
-    dplyr::mutate(sdRateDiff = sqrt((sdRate.x)^2 + (sdRate.y)^2)) %>% 
-    dplyr::mutate(meanElevDiff = meanElev.x - meanElev.y) %>% 
-    dplyr::mutate(sdElevDiff = sqrt((sdElev.x)^2 + (sdElev.y)^2)) 
-    # reorder summary columns
-   if(daily) {
-     summary_all %>% 
-       dplyr::select(GPS, Date,
-                     n.x, n.y, nDiff,
-                     meanLat.x, meanLat.y, meanLatDiff,
-                     sdLat.x, sdLat.y, sdLatDiff,
-                     meanLong.x, meanLong.y, meanLongDiff,
-                     sdLong.x, sdLong.y, sdLongDiff,
-                     meanDist.x, meanDist.y, meanDistDiff,
-                     sdDist.x, sdDist.y, sdDistDiff,
-                     meanCourse.x, meanCourse.y, meanCourseDiff,
-                     sdCourse.x, sdCourse.y, sdCourseDiff,
-                     meanRate.x, meanRate.y, meanRateDiff,
-                     sdRate.x, sdRate.y, sdRateDiff,
-                     meanElev.x, meanElev.y, meanElevDiff,
-                     sdElev.x, sdElev.y, sdElevDiff)
-   }
-   else {
-     summary_all %>% 
-     dplyr::select(1,
-                   n.x, n.y, nDiff,
-                   meanLat.x, meanLat.y, meanLatDiff,
-                   sdLat.x, sdLat.y, sdLatDiff,
-                   meanLong.x, meanLong.y, meanLongDiff,
-                   sdLong.x, sdLong.y, sdLongDiff,
-                   meanDist.x, meanDist.y, meanDistDiff,
-                   sdDist.x, sdDist.y, sdDistDiff,
-                   meanCourse.x, meanCourse.y, meanCourseDiff,
-                   sdCourse.x, sdCourse.y, sdCourseDiff,
-                   meanRate.x, meanRate.y, meanRateDiff,
-                   sdRate.x, sdRate.y, sdRateDiff,
-                   meanElev.x, meanElev.y, meanElevDiff,
-                   sdElev.x, sdElev.y, sdElevDiff)
-   }
+    dplyr::mutate(sdRateDiff = sqrt((sdRate.x)^2 + (sdRate.y)^2))
+  
+  if(use_elev) {
+    summary <- summary_all %>% 
+      dplyr::mutate(meanElevDiff = meanElev.x - meanElev.y) %>% 
+      dplyr::mutate(sdElevDiff = sqrt((sdElev.x)^2 + (sdElev.y)^2)) 
+  }
+   
+   return(summary_all)
 }
 
 #'
@@ -428,8 +427,8 @@ violin_compare <- function(df_summary, by, col_name, export = FALSE, out = NULL)
 #'Compares moving averages of a variable for two datasets over time, grouped by GPS
 #'GPS, Date, and col columns should match
 #'
-#'@param correct reference df
-#'@param candidate df to be compared to the reference
+#'@param correct reference data frame
+#'@param candidate data frame to be compared to the reference
 #'@param col variable to plot the moving average for
 #'@param export logical, whether to export plot, defaults to False
 #'@param out .png file name to save plot when export is True
@@ -472,27 +471,34 @@ line_compare <- function(correct, candidate, col, export = FALSE, out = NULL) {
 #'Compares two animal datasets and calculates daily summary statistics by GPS
 #'GPS, date, lat, long, course, distance, rate, elevation column names should match. 
 #'
-#'@param correct reference df
-#'@param candidate df to be compared to the reference
+#'@param correct reference data frame
+#'@param candidate data frame to be compared to the reference
 #'@param export logical, whether to export summary to .csv, defaults to False
 #'@param out desired file name of .csv output summary when export is True
-#'@return summary df
+#'@param use_elev logical, whether to include elevation in summary, defaults to true
+#'@return summary data frame
 #'@examples
 #'# Compare and summarise unfiltered demo cows to filtered, grouped by both Date and GPS
 #'
 #'compare_summarise_daily(demo_unfiltered_elev, demo_filtered_elev)
 #'@export
 #'
-compare_summarise_daily <- function(correct, candidate, export = TRUE, out = NULL) {
-  correct_summary <- correct %>% 
-    summarise_anidf(NULL, Latitude, Longitude, Distance, Course, Rate, Elevation, daily=TRUE)
-  
-  candidate_summary <- candidate %>% 
-    summarise_anidf(NULL, Latitude, Longitude, Distance, Course, Rate, Elevation, daily=TRUE)
-  
+compare_summarise_daily <- function(correct, candidate, use_elev = TRUE, export = FALSE, out = NULL) {
+  if(use_elev) {
+    correct_summary <- correct %>% 
+      summarise_anidf(NULL, Latitude, Longitude, Distance, Course, Rate, Elevation, daily=TRUE)
+    candidate_summary <- candidate %>% 
+      summarise_anidf(NULL, Latitude, Longitude, Distance, Course, Rate, Elevation, daily=TRUE)
+  }
+  else {
+    correct_summary <- correct %>% 
+      summarise_anidf(NULL, Latitude, Longitude, Distance, Course, Rate, use_elev=FALSE, daily=TRUE)
+    candidate_summary <- candidate %>% 
+      summarise_anidf(NULL, Latitude, Longitude, Distance, Course, Rate, use_elev=FALSE, daily=TRUE)
+  }
   summary_all <- join_summaries(correct_summary, candidate_summary, daily=TRUE)
   
-  if(export & !is.null(out)) {
+  if(export) {
     utils::write.csv(summary_all, out, row.names = FALSE)
   }
   
@@ -503,37 +509,60 @@ compare_summarise_daily <- function(correct, candidate, export = TRUE, out = NUL
 #'
 #'Joins and reformats two animal data frames for the purpose of flag comparison
 #'
-#'@param correct reference df
+#'@param correct reference data frame
 #'@param candidate df to be compared to the reference
-#'@param elev logical, whether to include elevation, defaults to true
-#'@param slope logical, whether to include slope, defaults to true
-#'@return joined and reformatted df
+#'@param use_elev logical, whether to include elevation in comparison, defaults to true
+#'@param use_slope logical, whether to include slope in comparison, defaults to true
+#'@param has_flags logical, whether correct data frame has predefined flags, defaults to false
+#'@param dropped_flag dropped flag column, must be defined when has_flags is true, otherwise null
+#'@return joined and reformatted data frame
 #'@examples
 #'# Join and reformat unfiltered demo data and filtered demo data
 #'
 #'compare_flags(demo_unfiltered_elev, demo_filtered_elev)
 #'@export
-compare_flags <- function(correct, candidate, elev = TRUE, slope = TRUE) {
-    correct <- correct %>% dplyr::mutate(DateTime = as.POSIXct(DateTime, format="%Y-%m-%d %H:%M:%S"))
-    candidate <- candidate %>% dplyr::mutate(DateTime = as.POSIXct(DateTime, format="%Y-%m-%d %H:%M:%S"))  
-    joined <- dplyr::full_join(correct, candidate, by=c( "DateTime", "GPS")) %>%
+compare_flags <- function(correct, candidate, use_elev = TRUE, use_slope = TRUE, has_flags = FALSE, dropped_flag = NULL) {
+    correct <- correct %>% 
+      dplyr::mutate(DateTime = as.POSIXct(DateTime, format="%Y-%m-%d %H:%M:%S")) %>% 
+      dplyr::group_by(GPS) %>% 
+      dplyr::distinct(DateTime, .keep_all = TRUE) %>% 
+      dplyr::ungroup()
+    candidate <- candidate %>% 
+      dplyr::mutate(DateTime = as.POSIXct(DateTime, format="%Y-%m-%d %H:%M:%S")) %>% 
+      dplyr::group_by(GPS) %>% 
+      dplyr::distinct(DateTime, .keep_all = TRUE) %>% 
+      dplyr::ungroup()
+    join <- dplyr::full_join(correct, candidate, by=c("DateTime", "GPS"))
+    join_select <- join %>% 
       dplyr::select(DateTime, GPS,
                     Latitude.x, Latitude.y, Longitude.x, Longitude.y,
                     Distance.x, Distance.y, Rate.x, Rate.y,
-                    Course.x, Course.y, Elevation.x, Elevation.y, Slope.x, Slope.y,
-                    RateFlag, CourseFlag, DistanceFlag, TotalFlags)
-    if(elev) {
-      joined <- dplyr::bind_cols(joined, joined %>% dplyr::select(Elevation.x, Elevation.y))
+                    Course.x, Course.y)
+    if(use_elev) {
+      join_select <- join_select %>% 
+        dplyr::bind_cols(join %>% dplyr::select(Elevation.x, Elevation.y))
     }
-    if(slope) {
-      joined <- dplyr::bind_cols(joined, joined %>% dplyr::select(Slope.x, Slope.y))
+    if(use_slope) {
+      join_select <- join_select %>% 
+        dplyr::bind_cols(join %>% dplyr::select(Slope.x, Slope.y))
     }
-    joined <- joined %>% 
-      dplyr::mutate( Date = as.Date(DateTime, format="%Y-%m-%d"),
-                     TimeDiff = NA,
-                     TimeDiffMins = NA,
-                     cumDist.x = NA,
-                     cumDist.y = NA
+    if(has_flags) {
+      join_select <- join_select %>%
+        dplyr::bind_cols(join %>% dplyr::select(RateFlag.x, RateFlag.y, 
+                                                CourseFlag.x, CourseFlag.y,
+                                                DistanceFlag.x, DistanceFlag.y,
+                                                TotalFlags.x, TotalFlags.y, {{dropped_flag}}))
+    }
+    else {
+      join_select <- join_select %>%
+        dplyr::bind_cols(join %>% dplyr::select(RateFlag, CourseFlag, DistanceFlag, TotalFlags))
+    }
+   
+    join_select <- join_select %>% dplyr::mutate( Date = as.Date(DateTime, format="%Y-%m-%d"),
+                   TimeDiff = NA,
+                   TimeDiffMins = NA,
+                   cumDist.x = NA,
+                   cumDist.y = NA
                    ) %>%
     dplyr::group_by(GPS, Date) %>% 
     dplyr::arrange(DateTime, .by_group = TRUE) %>% 
@@ -545,26 +574,31 @@ compare_flags <- function(correct, candidate, elev = TRUE, slope = TRUE) {
                   cumDist.y = cumsum(Distance.y),
                   
                   TimeDiff = ifelse((is.na(dplyr::lag(DateTime,1)) | as.numeric(difftime(DateTime, dplyr::lag(DateTime,1), units="mins")) > 100), 0, as.numeric(DateTime - dplyr::lag(DateTime,1))), 
-                  TimeDiffMins = ifelse(TimeDiff == 0, 0, as.numeric(difftime(DateTime, dplyr::lag(DateTime,1), units="mins"))),
-                  Dropped.x = ifelse(!is.na(Latitude.x), 0, 1),
-                  Dropped.y = ifelse((TotalFlags < 2 & !DistanceFlag), 0, 1)) %>% 
-    dplyr::ungroup()
-    return(as.data.frame(joined))
+                  TimeDiffMins = ifelse(TimeDiff == 0, 0, as.numeric(difftime(DateTime, dplyr::lag(DateTime,1), units="mins")))
+                  )
+    if(has_flags) {
+      join_select <- join_select %>%
+        dplyr::mutate(Dropped.y = ifelse((TotalFlags.x < 2 & !DistanceFlag.y), 0, 1))
+    }
+    else {
+      join_select <- join_select %>% 
+        dplyr::mutate(Dropped.x = ifelse(!is.na(Latitude.x), 0, 1),
+                      Dropped.y = ifelse((TotalFlags < 2 & !DistanceFlag), 0, 1)) 
+    }
+      
+    return(as.data.frame(join_select %>% dplyr::ungroup()))
 }
 
 
 #'
-#'Alternative implementation of the robust peak detection algorithm by van Brakel 2016 
+#'Alternative implementation of the robust peak detection algorithm by van Brakel 2014 
+#'Classifies data points with modified z-scores greater than max_score as outliers ccording to Iglewicz and Hoaglin 1993
 #'
 #'@param df_comparison output of compare_flags 
 #'@param lag width of interval to compute rolling median and MAD, defaults to 5
 #'@param max_score modified z-score cutoff to classify observations as outliers, defaults to 3.5
 #'@return df with classifications
 #'@export
-#'@examples
-#'# Join and reformat unfiltered demo data and filtered demo data
-#'
-#'detect_peak_modz(demo_comparison, lag = 5, max_score = 3.5)
 #'
 detect_peak_modz <- function(df_comparison, lag = 5, max_score = 3.5) {
   peak_comparison <- df_comparison %>% 
